function [time ptt] = compute_ptt(ppg1,ppg2,Fs,pMin)%,wL, wD, pMin, pMax)

% Computes PULSE TRANSIT TIME, i.e. the phase delay between the (assumed to
% have common frequency) pulsations of ppg1 and ppg2

% FUNCTION compute_pulse(ppg,Fs,wL, wD, pMin, pMax)
% ppg:  ppg or similar signal containing heart rate pulsations
% Fs:   sampling rate in samples per second
% wL:   window length in seconds
% wD:   window displacement in samples
% pMin: minimum pulse frequency in bpm
% pMax: maximum pulse frequency in bpm

wL=12;
wD=Fs;
% pMin=45;
pMax=200;

wL=wL*Fs;
w=hamming(wL);
% w=ones(wL,1);
wN=round((length(ppg1)-wL)/wD);
fN=Fs*60*5;
pMin=round(pMin*fN/Fs/60);
pMax=round(pMax*fN/Fs/60);
p1I=zeros(1,wN);
p2I=zeros(1,wN);
ptt1=zeros(1,wN);
ptt2=zeros(1,wN);

for ii=1:wN
    start=wD*(ii-1)+1;
    stop=start+wL-1;
    
    % find frequency peak in ppg1, as in compute_pulse.m
    ppg1SegmentWindowed=stdnrm(stdnrm(ppg1(start:stop)).*w');
    ppg1SegmentPSD=abs(fft(ppg1SegmentWindowed,fN)).^2;
    ppg1SegmentPSD([1:pMin pMax:end])=0;
    [~,p1I(ii)]=max(ppg1SegmentPSD);
    if ii>1
        while p1I(ii)>1.1*p1I(ii-1)||p1I(ii)<.9*p1I(ii-1)
            ppg1SegmentPSD(p1I(ii))=0;
            [~,p1I(ii)]=max(ppg1SegmentPSD);
        end
    end
    
    % find frequency peak in ppg2, as in compute_pulse.m
    ppg2SegmentWindowed=stdnrm(stdnrm(ppg2(start:stop)).*w');
    ppg2SegmentPSD=abs(fft(ppg2SegmentWindowed,fN)).^2;
    ppg2SegmentPSD([1:pMin pMax:end])=0;
    [~,p2I(ii)]=max(ppg2SegmentPSD);
    if ii>1
        while p2I(ii)>1.1*p2I(ii-1)||p2I(ii)<.9*p2I(ii-1)
            ppg2SegmentPSD(p2I(ii))=0;
            [~,p2I(ii)]=max(ppg2SegmentPSD);
        end
    end
    
    % take the angle of the fft's
    ppg1SegmentAngle=angle(fft(ppg1SegmentWindowed,fN));
    ppg2SegmentAngle=angle(fft(ppg2SegmentWindowed,fN));
    
    % find difference in angle at peak points. two versions of this
    % difference are found, assuming the peak point are not exactly the
    % same for the two signals.
    ptt1(ii)=(ppg1SegmentAngle(p1I(ii))-ppg2SegmentAngle(p1I(ii)))/Fs;
    ptt2(ii)=(ppg1SegmentAngle(p2I(ii))-ppg2SegmentAngle(p2I(ii)))/Fs;
    
end

% "averaging" ptt1 and ptt2. where "discontinuities" are found in one but
% not in the other, use other. where "discontinuities" are found in both,
% use previous value.
c=.5*(ptt1(1)+ptt2(1));
d1=diff(ptt1);
d2=diff(ptt2);
d3=zeros(size(d2));
s1=2*std(d1);
s2=2*std(d2);
for ii=1:length(d1)
    if all([abs(d1(ii))>s1 abs(d2(ii))<s2])
        d3(ii)=d2(ii);
    elseif all([abs(d2(ii))>s2 abs(d1(ii))<s1])
        d3(ii)=d1(ii);
    elseif all([abs(d2(ii))>s2 abs(d1(ii))>s1])
        d3(ii)=0;
    else
        d3(ii)=.5*(d1(ii)+d2(ii));
    end
end
ptt=[c cumsum(d3)+c];

time=.01*round(100*linspace(wL/Fs,wL/Fs+Fs*length(ptt)/wD,length(ptt)));

