fps=10; %frames per second
snapshot=4; %number of seconds to record

%construct webcam object 3*8bit RGB @ res 640*360
obj=videoinput('winvideo',1,'RGB24_640x360');
triggerconfig(obj, 'manual');
obj.FramesPerTrigger=snapshot*fps;
src=getselectedsource(obj);

%set properties of exposure etc.
%all auto-behavior set to off to avoid unwanted fluctuations,
%meaning settings must be adjusted to environment
src.BacklightCompensation = 'off';
src.ExposureMode = 'manual';
src.WhiteBalanceMode = 'manual';
src.Brightness = 130;
src.Exposure = -10;
src.WhiteBalance = 7500;
src.FocusMode = 'manual';
src.Focus = 18;
src.Sharpness = 20;
src.Saturation = 100;
src.FrameRate = num2str(fps);

%get 1-frame snapshot for face recognition
start(obj)
    tic,pause(2),toc,disp(' (pause 2 sec)')%time to stabilize
    trigger(obj)%sets startpoint of following recording
    frame=getsnapshot(obj);
stop(obj)

%face recognition works faster with a reduction in resolution
%reduce=4 used when face fills vertical space of frame
reduce=4;
img=double(rgb2gray(imresize(frame,1/reduce)));
Face=FaceDetect('haarcascade_frontalface_alt2.xml',img);
Face=reduce*Face;

%set RegionOfInterest to forehead
%(top middle 96*32 px segment of face recognition rectangle)
ROI=[Face(1)+.5*Face(3)-48 Face(2) 96 32];
obj.ROIPosition = ROI;

%show ROI on frame to confirm
figure(1), hold off
image(frame),axis equal,hold on
rectangle('position',ROI,'edgecolor','g')

%record snapshot*fps number of frames
start(obj)
    tic,pause(2),toc,disp(' (pause 2 sec)')%time to stabilize
    trigger(obj)%sets startpoint of following recording
    tic,frames=getdata(obj,snapshot*fps);toc,disp(' (data acq)')
stop(obj)

%average and choose green channel
a=squeeze(sum(sum(frames)));
g=a(2,:);

%compute PSD
n=512;
f=60*linspace(0,fps,n);
p=abs(fft(zmean(g),n)).^2/n;
f=f(1:end/2);p=p(1:end/2);
figure(2),plot(f,p),hold on

%find and show peak of PSD between 45 and 200 bpm
i45=find(f>=45,1,'first');
i200=find(f>=200,1,'first');
[v i]=max(p(i45:i200));
stem(f(i45+i-1),p(i45+i-1),'r')
disp(round(f(i45+i-1)))
hold off
