%%
clear all,close all

% initialize camera
fps=10; %frames per second
snapshot=2; %number of seconds to record
numFrames=snapshot*fps;

%construct webcam object 3*8bit RGB @ res 640*360
obj=videoinput('winvideo',1,'RGB24_640x360');
triggerconfig(obj, 'manual');
obj.FramesPerTrigger=numFrames;
src=getselectedsource(obj);

%set properties of exposure etc.
%all auto-behavior set to off to avoid unwanted fluctuations,
%meaning settings must be adjusted to environment
src.BacklightCompensation = 'off';
src.ExposureMode = 'manual';
src.WhiteBalanceMode = 'manual';
src.Brightness = 90;
src.Exposure = -9;
% src.WhiteBalance = 6800;
src.WhiteBalance = 7200;
% src.WhiteBalance = 7500;
src.FocusMode = 'manual';
src.Focus = 18;
src.Sharpness = 20;
src.Saturation = 100;
src.FrameRate = num2str(fps);
%preview(obj)

%%
%get 1-frame snapshot for face recognition
start(obj)
%     tic,pause(2),toc,disp(' (pause 2 sec)')%time to stabilize
    trigger(obj)%sets startpoint of following recording
    frame=getsnapshot(obj);
stop(obj)

%face recognition works faster with a reduction in resolution
reduce=2;
img=double(rgb2gray(imresize(frame,1/reduce)));
Face=FaceDetect('haarcascade_frontalface_alt2.xml',img);
Face=reduce*Face;

%set RegionOfInterest to expanded face-rectangle
%make sure dimensions are integer multiples of dx
dx=16; % x-dim of square
dy=dx; % y-dim of square
reducewidth=5/5;
increaseheight=6/4;
xres=dx*ceil(Face(3)*reducewidth/dx);
yres=dy*ceil(Face(4)*increaseheight/dy);
Face(1)=Face(1)+.5*(Face(3)-xres);
Face(2)=Face(2)+.5*(Face(4)-yres);
Face(3)=xres;
Face(4)=yres;
Face=round(Face);
obj.ROIPosition = Face;
frame=frame(Face(2):Face(2)+Face(4),Face(1):Face(1)+Face(3),:);
numBlocks=xres*yres/(dx*dy);

% create vector of square coordinates on form
% [x-pos top left, y-pos top left, dx, dy]
b=zeros(4,numBlocks);
b(1,:)=mod(1:dx:dx*numBlocks,xres);
b(2,:)=sort(mod(1:dy:dy*numBlocks,yres));
b(3,:)=dx*ones(1,numBlocks);
b(4,:)=dy*ones(1,numBlocks);

%%
%record numFrames number of frames
start(obj)
    tic,pause(2),toc,disp(' (pause 2 sec)')%time to stabilize
    trigger(obj)%sets startpoint of following recording
    tic,frames=getdata(obj,numFrames);toc,disp(' (data acq)')
stop(obj)
g=squeeze(frames(:,:,2,:));%2=green

%%
%compute average for each block and frame
G=zeros(numBlocks,numFrames);
for ii=1:numFrames
    for jj=1:numBlocks
        rect=g(b(2,jj):b(2,jj)+dy-1,b(1,jj):b(1,jj)+dx-1,ii);
        G(jj,ii)=sum(sum(rect))/(dx*dy);
    end
end
for ii=1:numBlocks, G(ii,:)=zmean(G(ii,:)); end
%%
close all

%compute PSDs
N=256;%2^nextpow2(numFrames);
f=60*linspace(0,fps,N);
p=abs(fft(G',N)).^2/N;
f=f(1:end/2);p=p(1:end/2,:);

%find peaks in interval (i45,i200)
i45=find(f>=45,1,'first');
i200=find(f>=150,1,'first');
for ii=1:numBlocks,[Y(ii) I(ii)]=max(p(i45:i200,ii));end
Ibpm=f(i45+I-1);
figure,hist(Ibpm(Ibpm~=0),500)

%removing histogram peaks at endpoints is found to improve result
I2=Ibpm(Ibpm~=min(Ibpm));
I2=I2(I2~=min(I2));
I2=I2(I2~=max(I2));
I2=I2(I2~=max(I2));

%FOUND PULSE
pulse=mode(I2);
disp(round(pulse))

%display blocks where pulse was detected correctly
Icorrectpulse=find(round(Ibpm)==round(pulse));
Iwrongpulse=find(round(Ibpm)~=round(pulse));
Y(Iwrongpulse)=0;
Ibest=find(Y>2*std(Y));
Igood=find(Y>1*std(Y));
Iok=find(Y>0*std(Y));

figure, hold off
image(frame),axis equal,hold on

for ii=Iok    
rectangle('position',b(:,ii),'facecolor','g')
end

for ii=Igood    
rectangle('position',b(:,ii),'facecolor','y')
end

for ii=Ibest   
rectangle('position',b(:,ii),'facecolor','r')
end

